#define LIBDEBIAN_INSTALLER_MAP_REAL

#include <debian-installer/config.h>

#include <debian-installer/exec.h>
#include <debian-installer/hash.h>
#include <debian-installer/list.h>
#include <debian-installer/log.h>
#include <debian-installer/mem.h>
#include <debian-installer/mem_chunk.h>
#include <debian-installer/parser.h>
#include <debian-installer/parser_rfc822.h>
#include <debian-installer/package.h>
#include <debian-installer/packages.h>
#include <debian-installer/release.h>
#include <debian-installer/slist.h>
#include <debian-installer/string.h>
#include <debian-installer/tree.h>
#include <debian-installer/types.h>
#include <debian-installer/utils.h>
#include <debian-installer/system/devfs.h>
#include <debian-installer/system/dpkg.h>
#include <debian-installer/system/packages.h>
#include <debian-installer/system/prebaseconfig.h>
#include <debian-installer/system/subarch.h>
#include <debian-installer/system/utils.h>

